Search.setIndex({envversion:43,terms:{all:0,code:0,consider:0,queri:0,overlap:0,follow:0,grainier:0,depend:0,send:0,slowest:0,introduc:0,sourc:0,string:0,deepzoom_view:0,condition:0,cool:0,contenttyp:[],level:0,list:0,upload:0,"try":0,maxint:0,div:0,smaller:0,enjoi:0,zero:0,design:0,pass:0,further:0,port:0,append:0,compat:0,sub:0,neg:0,section:0,uniform:0,delet:0,version:0,method:0,xml:0,elimin:0,full:0,behavior:0,gener:0,bodi:0,let:0,path:0,sinc:0,convert:0,convers:0,larger:0,within:0,chang:0,pillow:0,requestcontext:0,appli:0,modul:0,thangavelu:0,visibl:0,demonstr:0,total:0,from:0,sharper:0,visit:0,two:0,next:0,call:0,type:0,start:0,inflect:0,trail:0,deepzoom_div:0,none:0,setup:0,work:0,uniqu:0,antialia:0,purpos:0,root:0,def:0,overrid:0,nearest:0,tar:0,share:0,templat:0,tag:0,want:0,cours:0,end:0,programmat:0,write:0,anyon:0,tile:0,updat:0,dialog:0,overridden:0,after:0,befor:0,mai:0,associ:0,averag:0,circumst:0,orchestr:0,correspond:0,element:0,gap:0,allow:0,suggest:0,doesnotexist:0,order:0,checkbox:0,gasienica:0,becaus:0,jpeg:0,trade:0,uploaded_imag:0,hierarchi:0,mainli:0,paramet:0,superior:0,consensu:0,davidjcox:0,better:0,lossless:0,myimag:0,pixel:0,them:0,good:0,"return":0,halt:0,handl:0,overal:0,auth:0,now:0,nor:0,somewher:0,name:0,realist:0,edit:0,drop:0,separ:0,accommod:0,each:0,unicod:0,complet:0,side:0,replac:0,idea:0,contrib:0,"static":0,expect:0,extract:0,event:0,special:0,out:0,shown:0,influenc:0,daniel:0,urlpattern:0,insid:0,differ:0,standard:0,standalon:0,reason:0,base:0,releas:0,org:0,care:0,v_deepzoom:0,static_url:0,david:0,filter:0,thing:0,unabl:0,view:0,think:0,urlconf:0,origin:0,tile_format:0,rang:0,media_root:0,qualiti:0,number:0,yourself:0,alreadi:0,installed_app:0,size:0,given:0,script:0,messag:[],too:0,unfett:0,"final":0,store:0,imagefield:0,option:0,lower:0,specifi:0,task:0,part:0,attempt:0,deepzoom_root:0,png:0,photograph:0,provid:0,deepzoom_imag:0,project:0,str:0,result:0,pre:0,viewer:0,queryset:0,ani:0,packag:0,have:0,tabl:0,need:0,seem:0,incompat:0,test_set:0,min:0,also:0,contact:0,take:0,which:0,concern:0,bilinear:0,begin:0,sure:0,incorpor:0,object:0,compress:0,most:0,bsd:0,"class":0,collectstat:0,filesystem:0,url:0,request:0,doe:0,determin:0,pattern:0,usual:0,microsoft:0,text:0,session:[],involv:0,onli:0,locat:0,transact:0,should:0,gestur:0,tile_overlap:0,rice:0,get:0,pypi:0,increas:0,requir:0,enabl:0,integr:0,contain:0,bicub:0,attribut:0,summari:0,set:0,creator:0,see:0,sean:0,gmail:0,detect:0,databas:0,someth:0,handili:0,between:0,"import":0,neither:0,experi:0,approach:0,xvf:0,signatur:0,extend:0,javascript:0,addit:0,both:0,admin:0,cox:0,howev:0,etc:0,improv:0,com:0,load:0,point:0,instanti:0,wrap:0,height:0,seadragon:0,assum:0,uploadedimage_root:0,three:0,been:0,futur:0,dure:0,deeper:0,imag:0,overtax:0,argument:0,neighbor:0,craft:0,ident:0,passed_slug:0,defin:0,"while":0,unifi:0,abov:0,error:0,decreas:0,pertain:0,pyramid:0,site:0,jpg:0,deepzoom_j:0,develop:0,author:0,receiv:0,media:0,make:0,same:0,python:0,html:0,testrunn:0,higher:0,http:0,cubic:0,wherea:0,effect:0,render_to_respons:0,rais:0,initi:0,kapil:0,"_deepzoom_obj":0,off:0,markup:0,deepzoom_obj:0,inherit:0,exampl:0,expens:0,thi:0,choos:0,undefin:0,model:0,roughli:0,left:0,comment:0,just:0,image_qu:0,bandwidth:0,touch:0,speed:0,previous:0,easi:0,mix:0,had:0,except:0,add:0,save:0,app:0,match:0,build:0,vendor:0,format:0,know:0,you:0,saniti:0,knock:0,like:0,specif:0,anyth:0,zoom:0,slugifi:0,server:0,collect:0,necessari:0,either:0,output:0,resiz:0,manag:0,context_inst:0,creation:0,some:0,deepzoom_tag:0,resolv:0,sampl:0,staticfil:0,syncdb:0,successfulli:0,virtualenv:0,avoid:0,per:0,tile_s:0,refer:0,sensibl:0,run:0,usag:0,wget:0,slug:0,src:0,about:0,actual:0,page:0,modal:0,commit:[],produc:0,block:0,own:0,curiou:0,"float":0,appropri:0,empti:0,ensur:0,your:0,wai:0,question:0,"long":0,custom:0,avail:0,width:0,includ:0,suit:0,"function":0,forc:0,deepzoom_param:0,atom:0,line:0,highest:0,"throw":0,largest:0,consist:0,possibl:0,"default":0,wish:0,displai:0,below:0,limit:0,http404:0,embed:0,featur:0,pil:0,creat:0,"int":0,certain:0,"abstract":0,deep:0,exist:0,file:0,pip:0,check:0,when:0,prepend:0,field:0,whew:0,uploadedimag:0,autocommit:0,draw:0,intend:0,subdivid:0,prudent:0,reduc:0,backward:0,static_root:0,directori:0,tradeoff:0,potenti:0,time:0,resize_filt:0},objtypes:{},objnames:{},filenames:["index"],titles:["Django-deepzoom documentation"],objects:{},titleterms:{what:0,feedback:0,licens:0,configur:0,search:0,django:0,content:0,how:0,leav:0,can:0,deepzoom:0,test:0,"new":0,document:0,instal:0}})